//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  alias.cpp
//  This is a part of the alias source code.
//
//  Copyright (C) 2004 ilmcuts
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "alias.hpp"

using std::string;


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// typedefs
//
typedef std::map<string, string, stringicmp> AliasMap;


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Globals
//
HWND g_hMain = NULL;
HWND g_hLitestep = NULL;

const UINT g_lsMessages[] = { LM_GETREVID, LM_REFRESH, 0 };

AliasMap g_AliasMap;


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// initModuleEx
//
int initModuleEx(HWND hLiteStep, HINSTANCE hInst, LPCSTR /* pszPath */)
{
    g_hLitestep = hLiteStep;

    //
    // window (class) creation/registration
    //
    WNDCLASS wc = { 0 };
    wc.lpfnWndProc = WndProc;
    wc.hInstance = hInst;
    wc.lpszClassName = g_szAppName;

    if (!RegisterClass(&wc))
    {
        ErrorMessage(GetLastError());
    }

    g_hMain = CreateMessageWindow(hInst, g_szAppName, NULL);

    SetWindowLong(g_hMain, GWL_USERDATA, magicDWord);
    SendMessage(g_hLitestep, LM_REGISTERMESSAGE,
        (WPARAM)g_hMain, (LPARAM)g_lsMessages);


    AddBangCommand("!alias", BangAlias);
    LoadSetup(false);

    return 0;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// quitModule
//
void quitModule(HINSTANCE hInst)
{
    ClearSetup();
    RemoveBangCommand("!alias");
    
    if (g_hMain)
    {
        SendMessage(g_hLitestep, LM_UNREGISTERMESSAGE,
            (WPARAM)g_hMain, (LPARAM)g_lsMessages);
        
        DestroyWindow(g_hMain);

        g_hMain = NULL;
    }

    UnregisterClass(g_szAppName, hInst);

    g_hLitestep = NULL;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// LoadSetup
//
void LoadSetup(bool bIsRefresh)
{
    try
    {
        EnumConfig(NULL, "*Alias", &ParseAliasLine);
    }
    catch (std::exception& e)
    {
        ErrorMessage(e.what());
    }
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ClearSetup
//
void ClearSetup()
{
    for (AliasMap::iterator iter = g_AliasMap.begin(); iter != g_AliasMap.end(); ++iter)
    {
        RemoveBangCommand(iter->first.c_str());
    }
    
    g_AliasMap.clear();
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ParseAliasLine
//
// callback for EnumSettings in LoadSetup
// also used in BangAlias
//
bool ParseAliasLine(LPCSTR pszLine)
{
    if (pszLine != NULL)
    {
        LPCSTR pszRest = pszLine;
        char szName[MAX_LINE_LENGTH] = { 0 };
        
        if (GetToken(pszLine, szName, &pszRest, FALSE))
        {
            AliasMap::iterator iter = g_AliasMap.find(szName);
            
            if (iter != g_AliasMap.end())
            {
                iter->second.assign(pszRest);
            }
            else
            {
                g_AliasMap.insert(AliasMap::value_type(szName, pszRest));
                AddBangCommandEx(szName, BangAliasExec);
            }
        }
    }
    
    return true;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// FormatString
//
bool FormatString(char* pszBuffer, size_t cchBuffer, const char* pszFormat,
                  const char** ppszArray)
{
    return (0 != FormatMessage(
        FORMAT_MESSAGE_FROM_STRING | FORMAT_MESSAGE_ARGUMENT_ARRAY,
        pszFormat,
        0,
        0,
        pszBuffer,
        cchBuffer,
        (va_list*)ppszArray));
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// DeleteCharArray
//
// for std::for_each
//
static void DeleteCharArray(char* pszString)
{
    delete[] pszString;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// BangAlias
//
void BangAlias(HWND, LPCSTR pszArgs)
{
    ParseAliasLine(pszArgs);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// BangAliasExec
//
// the alias "interpreter"
//
void BangAliasExec(HWND, LPCSTR pszName, LPCSTR pszArgs)
{
    AliasMap::iterator iter = g_AliasMap.find(pszName);
    
    if (iter != g_AliasMap.end())
    {
        //
        // okay... this is really, really ugly
        // but i really, really can't think of a different way to do this =)
        //
        char* pCurrent = NULL;
        const char* pszNextToken = pszArgs;
        std::vector<char*> strVec;
        
        while (true)
        {
            pCurrent = new char[MAX_LINE_LENGTH];
            
            if (GetToken(pszNextToken, pCurrent, &pszNextToken, FALSE))
            {
                strVec.push_back(pCurrent);
            }
            else
            {
                delete[] pCurrent;
                break;
            }
        }
        
        char szResult[MAX_LINE_LENGTH] = { 0 };
        
        FormatString(szResult, MAX_LINE_LENGTH, iter->second.c_str(),
            const_cast<const char**>(&strVec[0]));
        
        std::for_each(strVec.begin(), strVec.end(), DeleteCharArray);
        
        LSExecute(NULL, szResult, SW_SHOWNORMAL);
    }
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// WndProc
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
    switch (uMessage)
    {
        case LM_GETREVID:
        {
            if (wParam == 0 || wParam == 1)
            {
                if (SUCCEEDED(StringCchPrintf((char*)lParam, 64,
                    "%s %s", g_szAppName, g_szAppVersion)))
                {
                    return lstrlen((char*)lParam);
                }
            }
            
            // something above failed
            lParam = NULL;
        }
        break;

        case LM_REFRESH:
        {
            ClearSetup();
            LoadSetup(true);

            return 0;
        }
        break;

        default:
        break;
    }

    return DefWindowProc(hWnd, uMessage, wParam, lParam);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// DllMain
//
// DLL entry point
//
BOOL APIENTRY DllMain(HINSTANCE hInst, DWORD dwReason, LPVOID /* pvReserved */)
{
    if (dwReason == DLL_PROCESS_ATTACH)
    {
        DisableThreadLibraryCalls(hInst);
    }
    
    return TRUE;
}
